@@ -166,9 +166,8 @@ class DOMHTMLMovieParser(DOMParserBase):
     extractors = [Extractor(label='title',
                             path="//h1",
                             attrs=Attribute(key='title',
-                                            path=".//text()",
-                                            postprocess=lambda x: \
-                                            analyze_title(x, canonical=1))),
+                                        path=".//text()",
+                                        postprocess=analyze_title)),
 
                 Extractor(label='glossarysections',
                         group="//a[@class='glossary']",
@@ -336,18 +335,15 @@ class DOMHTMLMovieParser(DOMParserBase):
                         path="../ul/li",
                         attrs=Attribute(key=None,
                                 multi=True,
-                                path={'text': ".//text()",
-                                        'comp-link': "./a/@href"},
+                                path={'name': "./a//text()",
+                                        'comp-link': "./a/@href",
+                                        'notes': "./text()"},
                                 postprocess=lambda x: \
-                                        Company(name=x.get('text', u''),
-                                companyID=analyze_imdbid(x.get('comp-link')))
+                                        Company(name=x.get('name') or u'',
+                                companyID=analyze_imdbid(x.get('comp-link')),
+                                notes=(x.get('notes') or u'').strip())
                             )),
 
-                #Extractor(label='votes and rating',
-                #        path="//div[@class='general rating']",
-                #        attrs=Attribute(key='votes and rating',
-                #                        path=".//text()")),
-
                 Extractor(label='votes and rating',
                         path="//div[@class='meta']",
                         attrs=Attribute(key='votes and rating',
@@ -526,7 +522,7 @@ class DOMHTMLPlotParser(DOMParserBase):
                             multi=True,
                             path={'plot': './text()',
                                 'author': './i/a/text()'},
-                            postprocess=lambda x: _process_plotsummary(x)))]
+                            postprocess=_process_plotsummary))]
 
 
 def _process_award(x):
@@ -584,7 +580,7 @@ class DOMHTMLAwardsParser(DOMParserBase):
                     'notes': "./small[last()]//text()",
                     'anchor': ".//text()"
                     },
-                postprocess=lambda x: _process_award(x)
+                postprocess=_process_award
                 )),
         Extractor(label='recipients',
             group="//table//big",
@@ -812,7 +808,6 @@ class DOMHTMLQuotesParser(DOMParserBase):
     """
     _defGetRefs = True
 
-    # FIXME: it still contains too many ::
     extractors = [
         Extractor(label='quotes',
             path="//div[@class='_imdbpy']",
@@ -828,6 +823,9 @@ class DOMHTMLQuotesParser(DOMParserBase):
             r'\1<div class="_imdbpy">'),
         (re.compile('<hr width="30%">', re.I), '</div>'),
         (re.compile('<hr/>', re.I), '</div>'),
+        (re.compile('<script.*?</script>', re.I|re.S), ''),
+        # For BeautifulSoup.
+        (re.compile('<!-- sid: t-channel : MIDDLE_CENTER -->', re.I), '</div>')
         ]
 
     def postprocess_data(self, data):
@@ -1171,10 +1169,17 @@ class DOMHTMLTechParser(DOMParserBase):
         for key in data:
             data[key] = filter(None, data[key])
         if self.kind in ('literature', 'business', 'contacts') and data:
+            if 'screenplay/teleplay' in data:
+                data['screenplay-teleplay'] = data['screenplay/teleplay']
+                del data['screenplay/teleplay']
             data = {self.kind: data}
         else:
+            if self.kind == 'publicity':
+                if 'biography (print)' in data:
+                    data['biography-print'] = data['biography (print)']
+                    del data['biography (print)']
             # Tech info.
-            for key in data:
+            for key in data.keys():
                 if key.startswith('film negative format'):
                     data['film negative format'] = data[key]
                     del data[key]
@@ -1317,6 +1322,8 @@ class DOMHTMLNewsParser(DOMParserBase):
                 path={
                     'title': "./text()",
                     'fromdate': "../following-sibling::p[1]/small//text()",
+                    # FIXME: sometimes (see The Matrix (1999)) <p> is found
+                    #        inside news text.
                     'body': "../following-sibling::p[2]//text()",
                     'link': "../..//a[text()='Permalink']/@href",
                     'fulllink': "../..//a[starts-with(text(), " \
@@ -1398,7 +1405,7 @@ class DOMHTMLAmazonReviewsParser(DOMParserBase):
                     'review': ".//text()",
                     'author': "./i[starts-with(text(), '--')]/text()"
                     },
-                postprocess=lambda x: _parse_review(x)))
+                postprocess=_parse_review))
         ]
 
     preprocessors = [
@@ -1462,7 +1469,7 @@ class DOMHTMLSalesParser(DOMParserBase):
                             'cover': "./ancestor::td[1]/../td[1]"\
                                     "/a[1]/img[1]/@src",
                             },
-                        postprocess=lambda x: _parse_merchandising_link(x))),
+                        postprocess=_parse_merchandising_link)),
         Extractor(label='others',
                     group="//span[@class='_info']/..",
                     group_key="./h5/a[1]/text()",
@@ -1475,7 +1482,7 @@ class DOMHTMLSalesParser(DOMParserBase):
                             'shop': "./preceding-sibling::a[1]/text()",
                             'description': ".//text()",
                             },
-                        postprocess=lambda x: _parse_merchandising_link(x)))
+                        postprocess=_parse_merchandising_link))
     ]
 
     preprocessors = [
@@ -1550,7 +1557,7 @@ class DOMHTMLEpisodesParser(DOMParserBase):
                                 path=".//title/text()"),
                         Attribute(key='series movieID',
                                 path=".//h1/a[@class='main']/@href",
-                                postprocess=lambda x: analyze_imdbid(x))
+                                postprocess=analyze_imdbid)
                     ]),
             Extractor(label='episodes',
                 group="//div[@class='_imdbpy']/h3",
@@ -1566,7 +1573,7 @@ class DOMHTMLEpisodesParser(DOMParserBase):
                         'oad': self._oad_path,
                         'plot': "./following-sibling::text()[1]"
                     },
-                    postprocess=lambda x: _build_episode(x)))]
+                    postprocess=_build_episode))]
         if self.kind == 'episodes cast':
             self.extractors += [
                 Extractor(label='cast',
@@ -1664,7 +1671,6 @@ class DOMHTMLFaqsParser(DOMParserBase):
     _defGetRefs = True
 
     # XXX: bsoup and lxml don't match (looks like a minor issue, anyway).
-    # FIXME: remove the :: ?
 
     extractors = [
         Extractor(label='faqs',
@@ -1698,12 +1704,18 @@ class DOMHTMLAiringParser(DOMParserBase):
         aparser = DOMHTMLAiringParser()
         result = aparser.parse(airing_html_string)
     """
+    _containsObjects = True
+
     extractors = [
         Extractor(label='series title',
             path="//title",
             attrs=Attribute(key='series title', path="./text()",
                             postprocess=lambda x: \
                                     x.replace(' - TV schedule', u''))),
+        Extractor(label='series id',
+            path="//h1/a[@href]",
+            attrs=Attribute(key='series id', path="./@href")),
+
         Extractor(label='tv airings',
             path="//tr[@class]",
             attrs=Attribute(key='airing',
@@ -1722,7 +1734,7 @@ class DOMHTMLAiringParser(DOMParserBase):
                     'channel': x.get('channel').strip(),
                     'link': x.get('link'),
                     'title': x.get('title'),
-                    'season': x.get('season')
+                    'season': (x.get('season') or '').strip()
                     }
                 ))
     ]
@@ -1730,19 +1742,30 @@ class DOMHTMLAiringParser(DOMParserBase):
     def postprocess_data(self, data):
         if len(data) == 0:
             return {}
+        seriesTitle = data['series title']
+        seriesID = analyze_imdbid(data['series id'])
         if data.has_key('airing'):
             for airing in data['airing']:
                 title = airing.get('title', '').strip()
                 if not title:
-                    airing.clear()
-                    continue
-                e = Movie(title='%s {%s}' % (data['series title'],
-                    airing['title']), movieID=analyze_imdbid(airing['link']))
+                    epsTitle = seriesTitle
+                    if seriesID is None:
+                        continue
+                    epsID = seriesID
+                else:
+                    epsTitle = '%s {%s}' % (data['series title'],
+                                            airing['title'])
+                    epsID = analyze_imdbid(airing['link'])
+                e = Movie(title=epsTitle, movieID=epsID)
                 airing['episode'] = e
                 del airing['link']
                 del airing['title']
+                if not airing['season']:
+                    del airing['season']
         if 'series title' in data:
             del data['series title']
+        if 'series id' in data:
+            del data['series id']
         if 'airing' in data:
             data['airing'] = filter(None, data['airing'])
         if 'airing' not in data or not data['airing']:
