@@ -27,7 +27,8 @@ import string
 from copy import copy, deepcopy
 from time import strptime, strftime
 
-from imdb._exceptions import IMDbParserError, IMDbError
+from imdb import VERSION
+from imdb._exceptions import IMDbParserError
 
 # The regular expression for the "long" year format of IMDb, like
 # "(1998)" and "(1986/II)", where the optional roman number (that I call
@@ -61,11 +62,21 @@ def canonicalName(name):
     #      - Jr.: 8025
     # Don't convert names already in the canonical format.
     if name.find(', ') != -1: return name
+    if isinstance(name, unicode):
+        joiner = u'%s, %s'
+        sur_joiner = u'%s %s'
+        sur_space = u' %s'
+        space = u' '
+    else:
+        joiner = '%s, %s'
+        sur_joiner = '%s %s'
+        sur_space = ' %s'
+        space = ' '
     sname = name.split(' ')
     snl = len(sname)
     if snl == 2:
         # Just a name and a surname: how boring...
-        name = '%s, %s' % (sname[1], sname[0])
+        name = joiner % (sname[1], sname[0])
     elif snl > 2:
         lsname = [x.lower() for x in sname]
         if snl == 3: _indexes = (0, snl-2)
@@ -75,36 +86,42 @@ def canonicalName(name):
             if lsname[index] not in _sname_suffixes: continue
             try:
                 # Build the surname.
-                surn = '%s %s' % (sname[index], sname[index+1])
+                surn = sur_joiner % (sname[index], sname[index+1])
                 del sname[index]
                 del sname[index]
                 try:
                     # Handle the "Jr." after the name.
                     if lsname[index+2].startswith('jr'):
-                        surn += ' %s' % sname[index]
+                        surn += sur_space % sname[index]
                         del sname[index]
                 except (IndexError, ValueError):
                     pass
-                name = '%s, %s' % (surn, ' '.join(sname))
+                name = joiner % (surn, space.join(sname))
                 break
             except ValueError:
                 continue
         else:
-            name = '%s, %s' % (sname[-1], ' '.join(sname[:-1]))
+            name = joiner % (sname[-1], space.join(sname[:-1]))
     return name
 
 def normalizeName(name):
     """Return a name in the normal "Name Surname" format."""
+    if isinstance(name, unicode):
+        joiner = u'%s %s'
+    else:
+        joiner = '%s %s'
     sname = name.split(', ')
     if len(sname) == 2:
-        name = '%s %s' % (sname[1], sname[0])
+        name = joiner % (sname[1], sname[0])
     return name
 
-def analyze_name(name, canonical=0):
+def analyze_name(name, canonical=None):
     """Return a dictionary with the name and the optional imdbIndex
     keys, from the given string.
-    If canonical is true, it tries to convert the  name to
-    the canonical "Surname, Name" format.
+
+    If canonical is None (default), the name is stored in its own style.
+    If canonical is True, the name is converted to canonical style.
+    If canonical is False, the name is converted to normal format.
 
     raise an IMDbParserError exception if the name is not valid.
     """
@@ -120,24 +137,31 @@ def analyze_name(name, canonical=0):
             name = name[:opi].rstrip()
     if not name:
         raise IMDbParserError, 'invalid name: "%s"' % original_n
-    if canonical:
-        name = canonicalName(name)
+    if canonical is not None:
+        if canonical:
+            name = canonicalName(name)
+        else:
+            name = normalizeName(name)
     res['name'] = name
     if imdbIndex:
         res['imdbIndex'] = imdbIndex
     return res
 
 
-def build_name(name_dict, canonical=0):
+def build_name(name_dict, canonical=None):
     """Given a dictionary that represents a "long" IMDb name,
     return a string.
-    If canonical is not set, the name is returned in the normal
-    "Name Surname" format.
+    If canonical is None (default), the name is returned in the stored style.
+    If canonical is True, the name is converted to canonical style.
+    If canonical is False, the name is converted to normal format.
     """
     name = name_dict.get('canonical name') or name_dict.get('name', '')
     if not name: return ''
-    if not canonical:
-        name = normalizeName(name)
+    if canonical is not None:
+        if canonical:
+            name = canonicalName(name)
+        else:
+            name = normalizeName(name)
     imdbIndex = name_dict.get('imdbIndex')
     if imdbIndex:
         name += ' (%s)' % imdbIndex
@@ -167,31 +191,52 @@ def build_name(name_dict, canonical=0):
 # I've left in the list 'i' (1939 vs 2151) and 'uno' (52 vs 56)
 # I'm not sure what '-al' is, and so I've left it out...
 #
-# List of articles:
+# List of articles in utf-8 encoding:
 _articles = ('the', 'la', 'a', 'die', 'der', 'le', 'el',
             "l'", 'il', 'das', 'les', 'i', 'o', 'ein', 'un', 'de', 'los',
             'an', 'una', 'las', 'eine', 'den', 'het', 'gli', 'lo', 'os',
             'ang', 'oi', 'az', 'een', 'ha-', 'det', 'ta', 'al-',
-            'mga', "un'", 'uno', 'ett', 'dem', 'egy', 'els', 'eines', u'\xcf',
-            u'\xc7', u'\xd4\xef', u'\xcf\xe9')
+            'mga', "un'", 'uno', 'ett', 'dem', 'egy', 'els', 'eines',
+            '\xc3\x8f', '\xc3\x87', '\xc3\x94\xc3\xaf', '\xc3\x8f\xc3\xa9')
+
+_unicodeArticles = tuple([art.decode('utf_8') for art in _articles])
 
 # Articles in a dictionary.
 _articlesDict = dict([(x, x) for x in _articles])
+# Unicode version.
+_unicodeArticlesDict = dict([(x, x) for x in _unicodeArticles])
 _spArticles = []
+# Variations with a trailing space.
 for article in _articles:
     if article[-1] not in ("'", '-'): article += ' '
     _spArticles.append(article)
+_spUnicodeArticles = []
+for article in _unicodeArticles:
+    if article[-1] not in ("'", '-'): article += u' '
+    _spUnicodeArticles.append(article)
+
+articlesDicts = (_articlesDict, _unicodeArticlesDict)
+spArticles = (_spArticles, _spUnicodeArticles)
 
 def canonicalTitle(title):
-    """Return the title in the canonic format 'Movie Title, The'."""
+    """Return the title in the canonic format 'Movie Title, The';
+    beware that it doesn't handle long imdb titles, but only the
+    title portion, without year[/imdbIndex] or special markup."""
+    isUnicode = isinstance(title, unicode)
     try:
-        if title.split(', ')[-1].lower() in _articlesDict: return title
-    except IndexError: pass
+        if title.split(', ')[-1].lower() in articlesDicts[isUnicode]:
+            return title
+    except IndexError:
+        pass
+    if isUnicode:
+        _format = u'%s, %s'
+    else:
+        _format = '%s, %s'
     ltitle = title.lower()
-    for article in _spArticles:
+    for article in spArticles[isUnicode]:
         if ltitle.startswith(article):
             lart = len(article)
-            title = '%s, %s' % (title[lart:], title[:lart])
+            title = _format % (title[lart:], title[:lart])
             if article[-1] == ' ': title = title[:-1]
             break
     ## XXX: an attempt using a dictionary lookup.
@@ -208,12 +253,21 @@ def canonicalTitle(title):
     return title
 
 def normalizeTitle(title):
-    """Return the title in the normal "The Title" format."""
+    """Return the title in the normal "The Title" format;
+    beware that it doesn't handle long imdb titles, but only the
+    title portion, without year[/imdbIndex] or special markup."""
+    isUnicode = isinstance(title, unicode)
     stitle = title.split(', ')
-    if len(stitle) > 1 and stitle[-1].lower() in _articlesDict:
+    if len(stitle) > 1 and stitle[-1].lower() in articlesDicts[isUnicode]:
         sep = ' '
         if stitle[-1][-1] in ("'", '-'): sep = ''
-        title = '%s%s%s' % (stitle[-1], sep, ', '.join(stitle[:-1]))
+        if isUnicode:
+            _format = u'%s%s%s'
+            _joiner = u', '
+        else:
+            _format = '%s%s%s'
+            _joiner = ', '
+        title = _format % (stitle[-1], sep, _joiner.join(stitle[:-1]))
     return title
 
 
@@ -263,15 +317,16 @@ def is_series_episode(title):
     return 0
 
 
-def analyze_title(title, canonical=None,
-                    canonicalSeries=0, canonicalEpisode=0,
-                    _emptyString=u''):
+def analyze_title(title, canonical=None, canonicalSeries=None,
+                    canonicalEpisode=None, _emptyString=u''):
     """Analyze the given title and return a dictionary with the
     "stripped" title, the kind of the show ("movie", "tv series", etc.),
     the year of production and the optional imdbIndex (a roman number
     used to distinguish between movies with the same title and year).
-    If canonical is true, the title is converted to the canonical
-    format.
+
+    If canonical is None (default), the title is stored in its own style.
+    If canonical is True, the title is converted to canonical style.
+    If canonical is False, the title is converted to normal format.
 
     raise an IMDbParserError exception if the title is not valid.
     """
@@ -373,8 +428,11 @@ def analyze_title(title, canonical=None,
         title = title[1:-1].strip()
     if not title:
         raise IMDbParserError, 'invalid title: "%s"' % original_t
-    if canonical:
-        title = canonicalTitle(title)
+    if canonical is not None:
+        if canonical:
+            title = canonicalTitle(title)
+        else:
+            title = normalizeTitle(title)
     # 'kind' is one in ('movie', 'episode', 'tv series', 'tv mini series',
     #                   'tv movie', 'video movie', 'video game')
     result['title'] = title
@@ -419,13 +477,14 @@ def _convertTime(title, fromPTDFtoWEB=1, _emptyString=u''):
     return title
 
 
-def build_title(title_dict, canonical=None, canonicalSeries=0,
-                canonicalEpisode=0, ptdf=0, _doYear=1, _emptyString=u''):
+def build_title(title_dict, canonical=None, canonicalSeries=None,
+                canonicalEpisode=None, ptdf=0, _doYear=1, _emptyString=u''):
     """Given a dictionary that represents a "long" IMDb title,
     return a string.
 
-    If canonical is not true, the title is returned in the
-    normal format.
+    If canonical is None (default), the title is returned in the stored style.
+    If canonical is True, the title is converted to canonical style.
+    If canonical is False, the title is converted to normal format.
 
     If ptdf is true, the plain text data files format is used.
     """
@@ -474,10 +533,13 @@ def build_title(title_dict, canonical=None, canonicalSeries=0,
                 episode_title += ')'
             episode_title = '{%s}' % episode_title
         return '%s %s' % (pre_title, episode_title)
-    title = title_dict.get('canonical title') or title_dict.get('title', '')
+    title = title_dict.get('title', '')
     if not title: return _emptyString
-    if not canonical:
-        title = normalizeTitle(title)
+    if canonical is not None:
+        if canonical:
+            title = canonicalTitle(title)
+        else:
+            title = normalizeTitle(title)
     if pre_title:
         title = '%s %s' % (pre_title, title)
     if kind in (u'tv series', u'tv mini series'):
@@ -598,12 +660,12 @@ def cmpMovies(m1, m2):
     if m1y > m2y: return -1
     if m1y < m2y: return 1
     # Ok, these movies have the same production year...
-    m1t = m1.get('canonical title', _last)
-    m2t = m2.get('canonical title', _last)
+    #m1t = m1.get('canonical title', _last)
+    #m2t = m2.get('canonical title', _last)
     # It should works also with normal dictionaries (returned from searches).
-    if m1t is _last and m2t is _last:
-        m1t = m1.get('title', _last)
-        m2t = m2.get('title', _last)
+    #if m1t is _last and m2t is _last:
+    m1t = m1.get('title', _last)
+    m2t = m2.get('title', _last)
     if m1t < m2t: return -1
     if m1t > m2t: return 1
     # Ok, these movies have the same title...
@@ -813,16 +875,16 @@ def _normalizeValue(value, withRefs=False, modFunct=None, titlesRefs=None,
     return value
 
 
-def _tag4TON(ton, addAccessSystem=False):
+def _tag4TON(ton, addAccessSystem=False, _containerOnly=False):
     """Build a tag for the given _Container instance;
     both open and close tags are returned."""
     tag = ton.__class__.__name__.lower()
     what = 'name'
     if tag == 'movie':
-        value = ton['long imdb title']
+        value = ton.get('long imdb title') or ton.get('title', '')
         what = 'title'
     else:
-        value = ton['long imdb name']
+        value = ton.get('long imdb name') or ton.get('name', '')
     value = _normalizeValue(value)
     extras = u''
     crl = ton.currentRole
@@ -849,9 +911,15 @@ def _tag4TON(ton, addAccessSystem=False):
         beginTag = u'<%s id="%s"' % (tag, theID)
         if addAccessSystem and ton.accessSystem:
             beginTag += ' access-system="%s"' % ton.accessSystem
-        beginTag += u'><%s>%s</%s>' % (what, value, what)
+        if not _containerOnly:
+            beginTag += u'><%s>%s</%s>' % (what, value, what)
+        else:
+            beginTag += u'>'
     else:
-        beginTag = u'<%s><%s>%s</%s>' % (tag, what, value, what)
+        if not _containerOnly:
+            beginTag = u'<%s><%s>%s</%s>' % (tag, what, value, what)
+        else:
+            beginTag = u'<%s>' % tag
     beginTag += extras
     if ton.notes:
         beginTag += u'<notes>%s</notes>' % _normalizeValue(ton.notes)
@@ -973,6 +1041,13 @@ def _seq2xml(seq, _l=None, withRefs=False, modFunct=None,
     return _l
 
 
+_xmlHead = u"""<?xml version="1.0"?>
+<!DOCTYPE %s SYSTEM "http://imdbpy.sf.net/dtd/imdbpy{VERSION}.dtd">
+
+"""
+_xmlHead = _xmlHead.replace('{VERSION}', VERSION.replace('.', ''))
+
+
 class _Container(object):
     """Base class for Movie, Person, Character and Company classes."""
      # The default sets of information retrieved.
@@ -1053,7 +1128,8 @@ class _Container(object):
         if not self.__role:
             # XXX: needed?  Just ignore it?  It's probably safer to
             #      ignore it, to prevent some bugs in the parsers.
-            raise IMDbError, "Can't set ID of an empty Character/Person object."
+            #raise IMDbError,"Can't set ID of an empty Character/Person object."
+            pass
         if not self._roleIsPerson:
             if not isinstance(roleID, (list, tuple)):
                 self.currentRole.characterID = roleID
@@ -1250,6 +1326,11 @@ class _Container(object):
         # it will be called by the _normalizeValue function.
         origModFunct = self.modFunct
         self.modFunct = modNull
+        # XXX: not totally sure it's a good idea, but could prevent
+        #      problems (i.e.: the returned string always contains
+        #      a DTD valid tag, and not something that can be only in
+        #      the keys_alias map).
+        key = self.keys_alias.get(key, key)
         try:
             withRefs = False
             if key in self.keys_tomodify and \
@@ -1271,7 +1352,8 @@ class _Container(object):
 
     def asXML(self):
         """Return a XML representation of the whole object."""
-        beginTag, endTag = _tag4TON(self, addAccessSystem=True)
+        beginTag, endTag = _tag4TON(self, addAccessSystem=True,
+                                    _containerOnly=True)
         resList = [beginTag]
         for key in self.keys():
             value = self.getAsXML(key)
@@ -1279,7 +1361,8 @@ class _Container(object):
                 continue
             resList.append(value)
         resList.append(endTag)
-        return u''.join(resList)
+        head = _xmlHead % self.__class__.__name__.lower()
+        return head + u''.join(resList)
 
     def _getitem(self, key):
         """Handle special keys."""
@@ -1402,16 +1485,16 @@ class _Container(object):
         return deepcopy(self)
 
 
-def flatten(seq, toDescend=(list, dict, tuple),
-            yieldDictKeys=0, onlyKeysType=(_Container,), scalar=None):
+def flatten(seq, toDescend=(list, dict, tuple), yieldDictKeys=0,
+            onlyKeysType=(_Container,), scalar=None):
     """Iterate over nested lists and dictionaries; toDescend is a list
     or a tuple of types to be considered non-scalar; if yieldDictKeys is
     true, also dictionaries' keys are yielded; if scalar is not None, only
     items of the given type(s) are yielded."""
-    if not isinstance(seq, toDescend):
-        if scalar is None or isinstance(seq, scalar):
-            yield seq
-    else:
+    if scalar is None or isinstance(seq, scalar):
+        yield seq
+    #else:
+    if isinstance(seq, toDescend):
         if isinstance(seq, (dict, _Container)):
             if yieldDictKeys:
                 # Yield also the keys of the dictionary.
@@ -1426,7 +1509,7 @@ def flatten(seq, toDescend=(list, dict, tuple),
                                 yieldDictKeys=yieldDictKeys,
                                 onlyKeysType=onlyKeysType, scalar=scalar):
                     yield v
-        else:
+        elif not isinstance(seq, (str, unicode, int, float)):
             for item in seq:
                 for i in flatten(item, toDescend=toDescend,
                                 yieldDictKeys=yieldDictKeys,
